@@ -487,11 +487,16 @@ lav_data_full <- function(data          = NULL,          # data.frame
         }
         #print( tracemem(X[[g]]) )
 
-        # standardize observed variables?
+        # standardize observed variables? numeric only!
         if(std.ov) {
-            X[[g]]  <- scale(X[[g]])[,] # three copies are made!
-            if(length(exo.idx) > 0L)
+            num.idx <- which(ov.names[[g]] %in% ov$name & ov$type == "numeric")
+            if(length(num.idx) > 0L) {
+                X[[g]][,num.idx]  <- scale(X[[g]][,num.idx,drop=FALSE])[,] 
+                # three copies are made!!!!!
+            }
+            if(length(exo.idx) > 0L) {
                 eXo[[g]] <- scale(eXo[[g]])[,]
+            }
         }
 
         # missing data
@@ -501,12 +506,20 @@ lav_data_full <- function(data          = NULL,          # data.frame
             # checking!
             if(length(Mp[[g]]$empty.idx) > 0L) {
                 X[[g]] <- X[[g]][-Mp[[g]]$empty.idx,,drop=FALSE]
+                # remove from case.idx
+                # idx <- which(case.idx[[g]] %in% Mp[[g]]$empty.idx)
+                empty.idx <- Mp[[g]]$empty.idx
+                empty.case.idx <- case.idx[[g]][empty.idx]
+                case.idx[[g]] <- case.idx[[g]][-empty.idx]
+                # remove from eXo
                 if(length(exo.idx) > 0L) {
-                    eXo[[g]] <- eXo[[g]][-Mp[[g]]$empty.idx,,drop=FALSE]
+                    eXo[[g]] <- eXo[[g]][-empty.idx,,drop=FALSE]
                 }
                 if(warn) {
-                    warning("lavaan WARNING: some cases are empty and will be removed:\n  ", paste(Mp[[g]]$empty.idx, collapse=" "))
+                    warning("lavaan WARNING: some cases are empty and will be removed:\n  ", paste(empty.case.idx, collapse=" "))
                 }
+                # give empty.idx case.idx? (for multiple groups):
+                Mp[[g]]$empty.idx <- empty.case.idx
             }
             if(warn && any(Mp[[g]]$coverage < 0.1)) {
                 warning("lavaan WARNING: due to missing values, some pairwise combinations have less than 10% coverage")
