@@ -205,14 +205,25 @@ lavParseModelString <- function(model.syntax = '', as.data.frame. = FALSE,
                     } else {
                         stop("lavaan ERROR: right-hand side of formula contains an intercept, but operator is \"", op, "\" in: ", x)
                     }
-                } else if(names(out)[j] == "zero") {
-                    if(op == "~") rhs.name <- ""
-                } else if(names(out)[j] == "constant") {
-                    if(op == "~") rhs.name <- ""
+                } else if(names(out)[j] == "zero" && op == "~") {
+                    rhs.name <- ""
+                } else if(names(out)[j] == "constant" && op == "~") {
+                    rhs.name <- ""
                 } else {
                     rhs.name <- names(out)[j]
                 }
 
+                # move this 'check' to post-parse 
+                #if(op == "|") {
+                #    th.name <- paste("t", j, sep="")
+                #    if(names(out)[j] != th.name) {
+                #        stop("lavaan ERROR: threshold ", j, " of variable ", 
+                #             sQuote(lhs.names[1]), " should be named ",
+                #             sQuote(th.name), "; found ", 
+                #             sQuote(names(out)[j]), "\n")
+                #    }
+                #}
+
                 # catch lhs = rhs and op = "=~"
                 if(op == "=~" && lhs.names[l] == names(out)[j]) {
                     stop("lavaan ERROR: latent variable `", lhs.names[l], "' can not be measured by itself")
